@@ -2,12 +2,14 @@ import errno
 import grp
 import json
 import os
+import psutil
 import pwd
+import shutil
+from jadi import component
 
-from aj.api import *
 from aj.api.http import url, HttpPlugin
-
-from aj.plugins.core.api.endpoint import endpoint, EndpointError, EndpointReturn
+from aj.api.endpoint import endpoint, EndpointError, EndpointReturn
+from aj.auth import authorize
 
 
 @component(HttpPlugin)
@@ -15,36 +17,87 @@ class Handler(HttpPlugin):
     def __init__(self, context):
         self.context = context
 
+    @url(r'/api/filesystem/mountpoints')
+    @authorize('filesystem:read')
+    @endpoint(api=True)
+    def handle_api_fs_mountpoints(self, http_context):
+        return [x.mountpoint for x in psutil.disk_partitions()]
+
     @url(r'/api/filesystem/read/(?P<path>.+)')
+    @authorize('filesystem:read')
     @endpoint(api=True)
     def handle_api_fs_read(self, http_context, path=None):
         if not os.path.exists(path):
-            raise EndpointReturn(404)
+            return http_context.respond_not_found()
         try:
-            return open(path).read()
+            content = open(path, 'rb').read()
+            if http_context.query:
+                encoding = http_context.query.get('encoding', None)
+                if encoding:
+                    content = content.decode(encoding)
+            return content
         except OSError as e:
-            raise EndpointError(e)
+            http_context.respond_server_error()
+            return json.dumps({'error': str(e)})
 
     @url(r'/api/filesystem/write/(?P<path>.+)')
+    @authorize('filesystem:write')
     @endpoint(api=True)
     def handle_api_fs_write(self, http_context, path=None):
         try:
+            content = http_context.body
+            if http_context.query:
+                encoding = http_context.query.get('encoding', None)
+                if encoding:
+                    content = content.decode('utf-8')
             with open(path, 'w') as f:
-                f.write(http_context.body)
+                f.write(content)
         except OSError as e:
             raise EndpointError(e)
 
-    @url(r'/api/filesystem/upload/(?P<path>.+)')
-    @endpoint(api=True)
-    def handle_api_fs_upload(self, http_context, path=None):
+    @url(r'/api/filesystem/upload')
+    @authorize('filesystem:write')
+    @endpoint(page=True)
+    def handle_api_fs_upload_chunk(self, http_context):
+        id = http_context.query['flowIdentifier']
+        chunk_index = http_context.query['flowChunkNumber']
+        chunk_dir = '/tmp/upload-%s' % id
         try:
-            with open(path, 'w') as f:
-                f.write(http_context.query['upload'])
-        except OSError as e:
-            raise EndpointError(e)
-        return True
+            os.makedirs(chunk_dir)
+        except:
+            pass
+        chunk_path = os.path.join(chunk_dir, chunk_index)
+
+        if http_context.method == 'GET':
+            if os.path.exists(chunk_path):
+                http_context.respond('200 OK')
+            else:
+                http_context.respond('204 No Content')
+        else:
+            with open(chunk_path, 'w') as f:
+                f.write(http_context.query['file'])
+            http_context.respond('200 OK')
+        return ''
+
+    @url(r'/api/filesystem/finish-upload')
+    @authorize('filesystem:write')
+    @endpoint(api=True)
+    def handle_api_fs_finish_upload(self, http_context):
+        name = http_context.json_body()['name']
+        path = http_context.json_body()['path']
+        id = http_context.json_body()['id']
+        chunk_dir = '/tmp/upload-%s' % id
+
+        target = os.path.join(path, name.replace('/', ''))
+        with open(target, 'wb') as f:
+            for i in range(len(os.listdir(chunk_dir))):
+                f.write(open(os.path.join(chunk_dir, str(i + 1))).read())
+
+        shutil.rmtree(chunk_dir)
+        return target
 
     @url(r'/api/filesystem/list/(?P<path>.+)')
+    @authorize('filesystem:read')
     @endpoint(api=True)
     def handle_api_fs_list(self, http_context, path=None):
         if not os.path.exists(path):
@@ -86,6 +139,7 @@ class Handler(HttpPlugin):
             raise EndpointError(e)
 
     @url(r'/api/filesystem/stat/(?P<path>.+)')
+    @authorize('filesystem:read')
     @endpoint(api=True)
     def handle_api_fs_stat(self, http_context, path=None):
         if not os.path.exists(path):
@@ -128,6 +182,7 @@ class Handler(HttpPlugin):
         return data
 
     @url(r'/api/filesystem/chmod/(?P<path>.+)')
+    @authorize('filesystem:write')
     @endpoint(api=True)
     def handle_api_fs_chmod(self, http_context, path=None):
         if not os.path.exists(path):
@@ -139,6 +194,7 @@ class Handler(HttpPlugin):
             raise EndpointError(e)
 
     @url(r'/api/filesystem/create-file/(?P<path>.+)')
+    @authorize('filesystem:write')
     @endpoint(api=True)
     def handle_api_fs_create_file(self, http_context, path=None):
         try:
@@ -147,6 +203,7 @@ class Handler(HttpPlugin):
             raise EndpointError(e)
 
     @url(r'/api/filesystem/create-directory/(?P<path>.+)')
+    @authorize('filesystem:write')
     @endpoint(api=True)
     def handle_api_fs_create_directory(self, http_context, path=None):
         try:
